package org.openxma.dsl.generator.helper;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

/**
 * Utility for extending a field
 * 
 */
public class FileUtil {

	/**
	 * 
	 * @param fileName
	 * @return
	 */
	public static Boolean fileExists(String fileName) {

		File f = new File(fileName);
		return f.exists();
	}

	/**
	 * 
	 * @param packageName
	 * @param className
	 * @return
	 */
	public static String classFileName(String packageName, String className) {
		return getPackagedFileName(packageName, className, "java");
	}

	/**
	 * 
	 * @param packageName
	 * @return
	 */
	public static String xsdFileName(String packageName) {
		return mkXsdFileName(packageName, null);
	}

	/**
	 * 
	 * @param packageName
	 * @return
	 */
	public static String xsdIncludeFileName(String packageName) {
		return mkXsdFileName(packageName, "-include");
	}

	/**
	 * 
	 * @param packageName
	 * @return
	 */
	public static String xsdIncludeFileNameWithoutPath(String packageName) {
		String name = xsdIncludeFileName(packageName);
		int p = name.lastIndexOf('/');
		if (p > -1)
			name = name.substring(p + 1);
		return name;
	}

	private static String mkXsdFileName(String packageName, String postfix) {
		final String ext = ".xsd";
		String lastNonVersionName = "default";
		if (packageName == null || packageName.length() < 1)
			return lastNonVersionName + ext;
		String[] packages = packageName.split("\\.");
		if (packages == null || packages.length < 1)
			return packageName + ext;
		StringBuilder fileName = new StringBuilder(70);
		for (String dir : packages) {
			fileName.append(dir);
			fileName.append("/");
			if (!isVersionString(dir))
				lastNonVersionName = dir;
		}
		String name = packageName.substring(packageName.lastIndexOf('.') + 1);
		fileName.append(lastNonVersionName);
		if (postfix != null)
			fileName.append(postfix);
		fileName.append(ext);
		return fileName.toString();
	}

	private static boolean isVersionString(String s) {
		if (s == null || s.length() < 2)
			return false;
		char c = s.charAt(0);
		if (Character.isDigit(c))
			return true;
		if (c != 'v' && c != 'V')
			return false;
		c = s.charAt(1);
		return Character.isDigit(c);
	}

	/**
	 * @param packageName
	 *            name of the package to resolve
	 * @param fileName
	 *            name of the file to append
	 * @param fileExtension
	 *            extension of the file to append
	 * @return the complete file path consisting of the
	 *         packageName+fileName+fileExtension
	 */
	public static String getPackagedFileName(String packageName, String fileName, String fileExtension) {

		StringBuffer filePath = new StringBuffer(70);
		if (packageName != null && packageName.length() > 0) {
			filePath.append(packageName.replace(".", "/"));
			filePath.append("/");
		}
		filePath.append(fileName);
		filePath.append(".");
		filePath.append(fileExtension);
		return filePath.toString();
	}

	/**
	 * @return an empty directory in the default temporary-file directory
	 * @throws IOException 
	 */
	public static File createTempDirectory() throws IOException {
		File tempDirectory = File.createTempFile("temp", Long.toString(System.nanoTime()));
		if (!(tempDirectory.delete())) {
			throw new IOException("Could not delete temp file: " + tempDirectory.getAbsolutePath());
		}
		if (!(tempDirectory.mkdir())) {
			throw new IOException("Could not create temp directory: " + tempDirectory.getAbsolutePath());
		}
		return tempDirectory;
	}

	/**
	 * Writes a string to a file creating the file if it does not exist.
	 * 
	 * @param content
	 *            the content to write to the file
	 * @param file the file to write into
	 * @return 
	 * @throws IOException 
	 */
	public static File writeStringToFile(String content, File file) throws IOException {
		write(content,openOutputStream(file));
		return file;
	}

	/**
	 * Opens a {@link FileOutputStream} for the specified file, checking and
	 * creating the parent directory if it does not exist.
	 * 
	 * @return a new {@link FileOutputStream} for the specified file
	 * @throws IOException
	 */
	public static FileOutputStream openOutputStream(File file) throws IOException {
		if (file.exists()) {
			if (file.isDirectory()) {
				throw new IOException("File '" + file + "' exists but is a directory");
			}
			if (file.canWrite() == false) {
				throw new IOException("File '" + file + "' cannot be written to");
			}
		} else {
			File parent = file.getParentFile();
			if (parent != null && parent.exists() == false) {
				if (parent.mkdirs() == false) {
					throw new IOException("File '" + file + "' could not be created");
				}
			}
		}
		return new FileOutputStream(file);
	}

	/**
	 * Writes chars from a <code>String</code> to bytes on an
	 * <code>OutputStream</code> using the default character encoding of the
	 * platform.
	 * <p>
	 * 
	 * @param data
	 *            the <code>String</code> to write, null ignored
	 * @param output
	 *            the <code>OutputStream</code> to write to
	 * @throws IOException
	 */
	public static void write(String data, OutputStream output) throws IOException {
		if (data != null) {
			output.write(data.getBytes());
		}
	}

}
